@@ -5,7 +5,7 @@
 ;;   Copyright (C) 2002,2003,2004,2005 Kiyoka Nishyama
 ;;   This program was derived from yc.el-4.0.13(auther: knak)
 ;;
-;;     $Date: 2005/09/21 14:12:15 $
+;;     $Date: 2005/10/05 12:45:08 $
 ;;
 ;; This file is part of Sumibi
 ;;
@@ -137,6 +137,9 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 			(cons 'sumibi-select-mode  sumibi-select-mode-map))
 		  minor-mode-map-alist)))
 
+;; $B%f!<%6!<3X=,<-=q(B
+(defvar sumibi-kakutei-history '())
+
 
 ;;;
 ;;; hooks
@@ -331,31 +334,43 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
     (let* (
 	   (yomi (buffer-substring-no-properties b e))
 	   (henkan-list (sumibi-henkan-request (sumibi-replace-keyword yomi))))
-
+      
       (if henkan-list
-	  (condition-case err
-	      (progn
-		(setq
-		 ;; $BJQ497k2L$NJ];}(B
-		 sumibi-henkan-list henkan-list
-		 ;; $BJ8@aA*Br=i4|2=(B
-		 sumibi-cand-n   (make-list (length henkan-list) 0)
-		 ;; 
-		 sumibi-cand-max (mapcar
-				  (lambda (x)
-				    (length x))
-				  henkan-list))
-
-		(sumibi-debug-print (format "sumibi-henkan-list:%s \n" sumibi-henkan-list))
-		(sumibi-debug-print (format "sumibi-cand-n:%s \n" sumibi-cand-n))
-		(sumibi-debug-print (format "sumibi-cand-max:%s \n" sumibi-cand-max))
-		;;
-		t)
-	    (sumibi-trap-server-down
-	     (beep)
-	     (message (error-message-string err))
-	     (setq sumibi-select-mode nil))
-	    (run-hooks 'sumibi-select-mode-end-hook))
+	  (progn
+	    ;; $BJ8@a$,0l$D$N>l9g$@$1%f!<%6!<<-=q$rMxMQ$9$k(B
+	    (when (>= 1 (length henkan-list))
+	      (let ( 
+		    (_ (assoc yomi sumibi-kakutei-history)))
+		(when _
+		  (setq henkan-list
+			(list
+			 (append
+			  `((j ,(cdr _) 0 0))
+			  (car henkan-list)))))))
+		
+	    (condition-case err
+		(progn
+		  (setq
+		   ;; $BJQ497k2L$NJ];}(B
+		   sumibi-henkan-list henkan-list
+		   ;; $BJ8@aA*Br=i4|2=(B
+		   sumibi-cand-n   (make-list (length henkan-list) 0)
+		   ;; 
+		   sumibi-cand-max (mapcar
+				    (lambda (x)
+				      (length x))
+				    henkan-list))
+		  
+		  (sumibi-debug-print (format "sumibi-henkan-list:%s \n" sumibi-henkan-list))
+		  (sumibi-debug-print (format "sumibi-cand-n:%s \n" sumibi-cand-n))
+		  (sumibi-debug-print (format "sumibi-cand-max:%s \n" sumibi-cand-max))
+		  ;;
+		  t)
+	      (sumibi-trap-server-down
+	       (beep)
+	       (message (error-message-string err))
+	       (setq sumibi-select-mode nil))
+	      (run-hooks 'sumibi-select-mode-end-hook)))
 	nil))))
 
 
@@ -451,11 +466,17 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 		(_n          (nth cnt sumibi-cand-n))
 		(_max        (nth cnt sumibi-cand-max))
 		(insert-word (cadr (nth _n x)))
+		(ank-word    (cadr (assoc 'l x)))
 		(_     
 		 (if (eq cnt sumibi-cand)
 		     (progn
-		      (insert insert-word)
-		      (message (format "[%s] candidate (%d/%d)" insert-word (+ _n 1) _max)))
+		       (insert insert-word)
+		       (message (format "[%s] candidate (%d/%d)" insert-word (+ _n 1) _max))
+		       ;; $B%f!<%6!<<-=q$KEPO?$9$k(B
+		       (setq sumibi-kakutei-history 
+			     (append
+			      `((,ank-word . ,insert-word))
+			      sumibi-kakutei-history)))
 		   (insert insert-word)))
 		(end         (point-marker))
 		(ov          (make-overlay start end)))
@@ -741,30 +762,68 @@ W/POuZ6lcg5Ktz885hZo+L7tdEy8W9ViH0Pd
 ;; $B%m!<%^;z4A;zJQ49;~!"JQ49BP>]$H$9$k%m!<%^;z$rFI$_Ht$P$94X?t(B
 (defun sumibi-skip-chars-backward ()
   (let* (
+	 (skip-chars
+	  (if auto-fill-function
+	      ;; auto-fill-mode $B$,M-8z$K$J$C$F$$$k>l9g2~9T$,$"$C$F$b(Bskip$B$rB3$1$k(B
+	      (concat sumibi-skip-chars "\n")
+	    ;; auto-fill-mode$B$,L58z$N>l9g$O$=$N$^$^(B
+	    sumibi-skip-chars))
+	    
 	 ;; $B%^!<%/$5$l$F$$$k0LCV$r5a$a$k!#(B
 	 (pos (or (and (markerp (mark-marker)) (marker-position (mark-marker)))
 		  1))
+
 	 ;; $B>r7o$K%^%C%A$9$k4V!"A0J}J}8~$K%9%-%C%W$9$k!#(B
 	 (result (save-excursion
-		   (skip-chars-backward sumibi-skip-chars (and (< pos (point)) pos))))
-	 (indent 0))
+		   (skip-chars-backward skip-chars (and (< pos (point)) pos))))
+	 (limit-point 0))
 
-    ;; $B%$%s%G%s%H0LCV$r5a$a$k!#(B
-    (save-excursion
-      (goto-char (point-at-bol))
-      (setq indent (skip-chars-forward (concat "\t " sumibi-stop-chars) (point-at-eol))))
+    (if auto-fill-function
+	;; auto-fill-mode$B$,M-8z$N;~(B
+	(progn
+	  (save-excursion
+	    (backward-paragraph)
+	    (when (< 1 (point))
+	      (forward-line 1))
+	    (goto-char (point-at-bol))
+	    (let (
+		  (start-point (point)))
+	      (setq limit-point
+		    (+
+		     start-point
+		     (skip-chars-forward (concat "\t " sumibi-stop-chars) (point-at-eol))))))
 
-    (sumibi-debug-print (format "(point) = %d  result = %d  indent = %d\n" (point) result indent))
-    (sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) (+ (point-at-bol) indent)))
+	  (sumibi-debug-print (format "(point) = %d  result = %d  limit-point = %d\n" (point) result limit-point))
+	  (sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) limit-point))
 
-    (if (< (+ (point) result)
-	   (+ (point-at-bol) indent))
-	;; $B%$%s%G%s%H0LCV$G%9%H%C%W$9$k!#(B
-	(- 
-	 (+ (point-at-bol) indent)
-	 (point))
+	  ;; $B%Q%i%0%i%U0LCV$G%9%H%C%W$9$k(B
+	  (if (< (+ (point) result) limit-point)
+	      (- 
+	       limit-point
+	       (point))
+	    result))
+
+      ;; auto-fill-mode$B$,L58z$N;~(B
+      (progn
+	(save-excursion
+	  (goto-char (point-at-bol))
+	  (let (
+		(start-point (point)))
+	    (setq limit-point
+		  (+ 
+		   start-point
+		   (skip-chars-forward (concat "\t " sumibi-stop-chars) (point-at-eol))))))
+
+	(sumibi-debug-print (format "(point) = %d  result = %d  limit-point = %d\n" (point) result limit-point))
+	(sumibi-debug-print (format "a = %d b = %d \n" (+ (point) result) limit-point))
+
+	(if (< (+ (point) result) limit-point)
+	    ;; $B%$%s%G%s%H0LCV$G%9%H%C%W$9$k!#(B
+	    (- 
+	     limit-point
+	     (point))
+	  result)))))
 
-      result)))
   
 ;;;
 ;;; with viper
@@ -893,7 +952,7 @@ point $B$+$i9TF,J}8~$KF1<o$NJ8;zNs$,B3$/4V$r4A;zJQ49$7$^$9!#(B
 (setq default-input-method "japanese-sumibi")
 
 (defconst sumibi-version
-"0.5.0"
+"0.5.1"
   )
 (defun sumibi-version (&optional arg)
   "$BF~NO%b!<%IJQ99(B"
