@@ -3,7 +3,7 @@
 # "SumibiWebApiSample.pl" is a sample program.
 #
 #   Copyright (C) 2005 Kiyoka Nishyama
-#     $Date: 2005/08/24 14:21:30 $
+#     $Date: 2005/09/25 14:44:00 $
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -33,7 +33,7 @@ my $ie    = "utf-8";
 my $oe    = "utf-8";
 
 
-my $sumibi = SOAP::Lite -> service("http://sumibi.org/sumibi/Sumibi_stable.wsdl");
+my $sumibi = SOAP::Lite -> service("http://sumibi.org/sumibi/Sumibi_testing.wsdl");
 #my $sumibi = SOAP::Lite -> service("http://sumibi.org/test/Sumibi.wsdl");
 
 #
